/**
 *
 *  \file utils.h
 *
 *  \date Created on: 9-feb-2009
 *  \author Author: Claudio Salvadori and Christian Nastasi
 *  \brief Uart communication files
 */
#include "ee.h"
#include "cpu/pic30/inc/ee_irqstub.h"

/******************************************************************************/
/*                         Forward Declarations                               */
/******************************************************************************/
/**
* @name Uart status
* \brief Different status of the Uart
*/
typedef enum {
	Status0 	= 0x00,	/**< Start */
	Status1		= 0x01, /**< Waiting for configuration msg*/
	Status2 	= 0x02,	/**< Configuration */
	Status3 	= 0x03,	/**< Capture */
} UART_Status;
/******************************************************************************/
/*                            INLINE Functions                                */
/******************************************************************************/
/* ----------------------------------------------------------------------------
|  Initializes UART1                                                           |
|                                                                              |
|  Setup UART1 interface (40MIPS only) in polling mode for both TX and RX      |
 ---------------------------------------------------------------------------- */
/**
* @name Initializes UART1
* \brief Setup UART1 interface (40MIPS only) in polling mode for both TX and RX
*/
__INLINE__
void
uart1_init(unsigned long int baud)
{
	/* ---------- */
	/* Stop UART1 */
	/* ---------- */
	U1MODEbits.UARTEN = 0;	/* Stop UART1 port */

	IEC0bits.U1RXIE = 0;	/* Disable RX Interrupts */
	IEC0bits.U1TXIE = 0;	/* Disable TX Interrupts */

	IFS0bits.U1RXIF = 0;	/* Clear RX Interrupt flag bits */
	IFS0bits.U1TXIF = 0;	/* Clear TX Interrupt flag bits */

	/* --------------- */
	/* Configure UART1 */
	/* --------------- */

	/* Set IO pins */
	TRISFbits.TRISF2 = 1;	/* Set IO pin: in for UART1 RX */
	TRISFbits.TRISF3 = 0;	/* Set IO pin: out for UART1 TX */


	U1BRG  = (2500000ul / baud) - 1; /* baud rate */

	/* Operation settings and start port */
	U1MODE = 0;			/* Reset UART1 Mode Register */
	U1MODEbits.UARTEN = 1;		/* Enable UART1 */

	U1STA = 0;			/* Reset UART1 Status Control Register*/
	U1STAbits.UTXEN = 1;		/* Enable UART1 TX pin */
}

/* ----------------------------------------------------------------------------
|  Send via UART1                                                              |
|                                                                              |
|  UART1 TX in polling mode                                                    |
 ---------------------------------------------------------------------------- */
/**
* @name Send via UART1
* \brief UART1 TX in polling mode
*/
__INLINE__
void
uart1_send(unsigned char data)
{
	while (U1STAbits.UTXBF);
	U1TXREG = data;
	while(!U1STAbits.TRMT);
}

/* ----------------------------------------------------------------------------
|  Receive  via UART1                                                          |
|                                                                              |
|  UART1 RX in polling mode                                                    |
 ---------------------------------------------------------------------------- */
/**
* @name Receive via UART1
* \brief UART1 RX in polling mode
*/
__INLINE__
unsigned char
uart1_receive(unsigned char* data)
{
	if (U1STAbits.OERR) {
		U1STAbits.OERR = 0;
		return 0;
	}

	if (U1STAbits.URXDA) {
		*data = U1RXREG & 0x00FF;
		return 1;
	} else {
		*data = 0;
		return 0;
	}

	return 0;
}

/* ----------------------------------------------------------------------------
|  Initializes UART2                                                           |
|                                                                              |
|  Setup UART2 interface (40MIPS only) in polling mode for both TX and RX.     |
 ---------------------------------------------------------------------------- */
/**
* @name Initializes UART2
* \brief Setup UART2 interface (40MIPS only) in polling mode for both TX and RX.
*/
__INLINE__
void
uart2_init(unsigned long int baud)
{
	/* ---------- */
	/* Stop UART2 */
	/* ---------- */
	U2MODEbits.UARTEN = 0;	/* Stop UART2 port */

	IEC1bits.U2RXIE = 0;	/* Disable RX Interrupts */
	IEC1bits.U2TXIE = 0;	/* Disable TX Interrupts */

	IFS1bits.U2RXIF = 0;	/* Clear RX Interrupt flag bits */
	IFS1bits.U2TXIF = 0;	/* Clear TX Interrupt flag bits */

	/* --------------- */
	/* Configure UART2 */
	/* --------------- */

	/* Set IO pins */
	TRISFbits.TRISF4 = 1;	/* Set IO pin: in for UART2 RX */
	TRISFbits.TRISF5 = 0;	/* Set IO pin: out for UART2 TX */


	U2BRG  = (2500000ul / baud) - 1; /* baud rate */

	/* Operation settings and start port */
	U2MODE = 0;			/* Reset UART2 Mode Register */
	U2MODEbits.UARTEN = 1;		/* Enable UART2 */

	U2STA = 0;			/* Reset UART2 Status Control Register*/
	U2STAbits.UTXEN = 1;		/* Enable UART2 TX pin */
}

/* ----------------------------------------------------------------------------
|  Send via UART2                                                              |
|                                                                              |
|  UART2 TX in polling mode                                                    |
 ---------------------------------------------------------------------------- */
/**
* @name Send via UART2
* \brief UART2 TX in polling mode
*/
__INLINE__
void
uart2_send(unsigned char data)
{
	while (U2STAbits.UTXBF);
	U2TXREG = data;
	while(!U2STAbits.TRMT);
}

/* ----------------------------------------------------------------------------
|  Receive  via UART2                                                          |
|                                                                              |
|  UART2 RX in polling mode                                                    |
 ---------------------------------------------------------------------------- */
/**
* @name Receive  via UART2
* \brief UART2 RX in polling mode
*/
__INLINE__
unsigned char
uart2_receive(unsigned char* data)
{
	if (U2STAbits.OERR) {
		U2STAbits.OERR = 0;
		return 0;
	}

	if (U2STAbits.URXDA) {
		*data = U2RXREG & 0x00FF;
		return 1;
	} else {
		*data = 0;
		return 0;
	}

	return 0;
}

/* ----------------------------------------------------------------------------
|  Initialies PC RS232                                                         |
|                                                                              |
|  Check which UART to use                                                     |
 ---------------------------------------------------------------------------- */
/**
* @name Initialies PC RS232
* \brief Check which UART to use
*/
__INLINE__
void
pc_serial_init(unsigned long int baud_rate)
{
	#ifdef PC_SERIAL_UART1
  	uart1_init(baud_rate);
	#else
  	uart2_init(baud_rate);
	#endif
}

/* ----------------------------------------------------------------------------
|  Read bytes from PC RS232                                                    |
|                                                                              |
 ---------------------------------------------------------------------------- */
/**
* @name Read bytes from PC RS232
*/
__INLINE__
unsigned char
pc_serial_read(unsigned char *data)
{
	#ifdef PC_SERIAL_UART1
	uart1_receive(data++);
	#else
	uart2_receive(data++);
	#endif

	return 1;
}

/* ----------------------------------------------------------------------------
|  Write bytes to PC RS232                                                     |
|                                                                              |
 ---------------------------------------------------------------------------- */
/**
* @name Write bytes to PC RS232
*/
__INLINE__
unsigned char
pc_serial_write(const unsigned char *data, unsigned int data_size)
{
	unsigned int i;
	for(i=0; i<data_size; i++){
		#ifdef PC_SERIAL_UART1
		uart1_send(*(data++));

		#else
		uart2_send(*(data++));
		#endif

	}
	return 1;
}



